@@ -1,10 +1,7 @@
 #!/usr/bin/env bash
 
 set -e
-
-if [ -n "${EZ_VERBOSE}" ]; then
-    set -x
-fi
+set -x
 
 # Warning: This variable API is experimental so these variables may be subject
 # to change in the future.
@@ -15,27 +12,33 @@ datadir=${datadir:=${prefix}/share}
 real_name=${real_name:=puppetserver}
 projdatadir=${projdatadir:=${datadir}/${real_name}}
 confdir=${confdir:=/etc}
-projconfdir=${projconfdir:=${confdir}/${real_name}}
-rundir=${rundir:=/var/run}
-bindir=${bindir:=/usr/bin}
+projconfdir=${projconfdir:=${confdir}/puppetlabs/${real_name}}
+rundir=${rundir:=/var/run/puppetlabs/${real_name}}
+# Application specific bin directory
+bindir=${bindir:=/opt/puppetlabs/server/apps/${real_name}/bin}
+# User facing bin directory, expected to be added to interactive shell PATH
+uxbindir=${uxbindir:=/opt/puppetlabs/bin}
+# symlinks of server binaries
+symbindir=${symbindir:=/opt/puppetlabs/server/bin}
 localstatedir=${localstatedir:=/var}
+app_prefix=${app_prefix:=/opt/puppetlabs/server/apps/${real_name}}
+app_data=${app_data:=/opt/puppetlabs/server/data/${real_name}}
+app_logdir=${app_logdir:=/var/log/puppetlabs/${real_name}}
 
 ##################
 # EZBake Vars    #
 ##################
 
-if [ -n "${EZ_VERBOSE}" ]; then
-    set +x
-    echo "#-------------------------------------------------#"
-    echo "The following variables are set: "
-    echo
-    env | sort
+set +x
+echo "#-------------------------------------------------#"
+echo "The following variables are set: "
+echo
+env | sort
 
-    echo
-    echo "End of variable print."
-    echo "#-------------------------------------------------#"
-    set -x
-fi
+echo
+echo "End of variable print."
+echo "#-------------------------------------------------#"
+set -x
 
 ##################
 # Task functions #
@@ -102,45 +105,44 @@ function task_install_source_deb {
 # Install the ezbake package software. This step is used during RPM &
 # Debian packaging during the 'install' phases.
 function task_install {
-    install -d -m 0755 "${DESTDIR}${projdatadir}"
-    install -m 0644 puppet-server-release.jar "${DESTDIR}${projdatadir}"
-    install -m 0774 ext/ezbake-functions.sh "${DESTDIR}${projdatadir}"
-    install -m 0644 ext/ezbake.manifest "${DESTDIR}${projdatadir}"
+    install -d -m 0755 "${DESTDIR}${app_prefix}"
+    install -d -m 0755 "${DESTDIR}${app_data}"
+    install -m 0644 puppet-server-release.jar "${DESTDIR}${app_prefix}"
+    install -m 0774 ext/ezbake-functions.sh "${DESTDIR}${app_prefix}"
+    install -m 0644 ext/ezbake.manifest "${DESTDIR}${app_prefix}"
     install -d -m 0755 "${DESTDIR}${projconfdir}/conf.d"
-    install -m 0644 ext/config/conf.d/ca.conf "${DESTDIR}${projconfdir}/conf.d/ca.conf"
     install -m 0644 ext/config/conf.d/puppetserver.conf "${DESTDIR}${projconfdir}/conf.d/puppetserver.conf"
+    install -m 0644 ext/config/conf.d/ca.conf "${DESTDIR}${projconfdir}/conf.d/ca.conf"
     install -m 0644 ext/config/request-logging.xml "${DESTDIR}${projconfdir}/request-logging.xml"
     install -m 0644 ext/config/logback.xml "${DESTDIR}${projconfdir}/logback.xml"
     install -m 0644 ext/config/bootstrap.cfg "${DESTDIR}${projconfdir}/bootstrap.cfg"
     install -m 0644 ext/config/conf.d/global.conf "${DESTDIR}${projconfdir}/conf.d/global.conf"
     install -m 0644 ext/config/conf.d/web-routes.conf "${DESTDIR}${projconfdir}/conf.d/web-routes.conf"
     install -m 0644 ext/config/conf.d/webserver.conf "${DESTDIR}${projconfdir}/conf.d/webserver.conf"
-    install -d -m 0755 "${DESTDIR}${projdatadir}/scripts"
-    install -m 0755 install.sh "${DESTDIR}${projdatadir}/scripts"
-    install -d -m 0755 "${DESTDIR}${projdatadir}/cli"
-    install -d -m 0755 "${DESTDIR}${projdatadir}/cli/apps"
+    install -d -m 0755 "${DESTDIR}${app_prefix}/scripts"
+    install -m 0755 install.sh "${DESTDIR}${app_prefix}/scripts"
+    install -d -m 0755 "${DESTDIR}${app_prefix}/cli"
+    install -d -m 0755 "${DESTDIR}${app_prefix}/cli/apps"
     install -d -m 0755 "${DESTDIR}${bindir}"
     install -m 0755 "ext/bin/${real_name}" "${DESTDIR}${bindir}/${real_name}"
-    install -m 0755 ext/cli/irb "${DESTDIR}${projdatadir}/cli/apps/irb"
-    install -m 0755 ext/cli/ruby "${DESTDIR}${projdatadir}/cli/apps/ruby"
-    install -m 0755 ext/cli/foreground "${DESTDIR}${projdatadir}/cli/apps/foreground"
-    install -m 0755 ext/cli/gem "${DESTDIR}${projdatadir}/cli/apps/gem"
+    install -d -m 0755 "${DESTDIR}${symbindir}"
+    ln -s "../apps/${real_name}/bin/${real_name}" "${DESTDIR}${symbindir}/${real_name}"
+    install -d -m 0755 "${DESTDIR}${uxbindir}"
+    ln -s "../server/apps/${real_name}/bin/${real_name}" "${DESTDIR}${uxbindir}/${real_name}"
+    install -m 0755 ext/cli/irb "${DESTDIR}${app_prefix}/cli/apps/irb"
+    install -m 0755 ext/cli/foreground "${DESTDIR}${app_prefix}/cli/apps/foreground"
+    install -m 0755 ext/cli/gem "${DESTDIR}${app_prefix}/cli/apps/gem"
+    install -m 0755 ext/cli/ruby "${DESTDIR}${app_prefix}/cli/apps/ruby"
     install -d -m 0755 "${DESTDIR}${rundir}"
-    install -d -m 700 "${DESTDIR}${localstatedir}/log/puppetserver"
+    install -d -m 700 "${DESTDIR}${app_logdir}"
 }
 
 function task_install_redhat {
     task install
-    echo "os-settings: {"                         > $DESTDIR/$projconfdir/conf.d/os-settings.conf
-    echo "    ruby-load-path: [$rubylibdir]"      >> $DESTDIR/$projconfdir/conf.d/os-settings.conf
-    echo "}"                                      >> $DESTDIR/$projconfdir/conf.d/os-settings.conf
 }
 
 function task_install_deb {
     task install
-    echo "os-settings: {"                       > $DESTDIR/$projconfdir/conf.d/os-settings.conf
-    echo "    ruby-load-path: [$rubylibdir]"    >> $DESTDIR/$projconfdir/conf.d/os-settings.conf
-    echo "}"                                    >> $DESTDIR/$projconfdir/conf.d/os-settings.conf
 }
 
 
@@ -188,10 +190,8 @@ function task_preinst_redhat {
            groupadd -r puppet || :
     # Add puppet user
     getent passwd puppet > /dev/null || \
-           useradd -r -g puppet -d ${datadir}/puppetserver -s /sbin/nologin \
+           useradd -r -g puppet -d "${app_data}" -s /sbin/nologin \
            -c "puppetserver daemon"  puppet || :
-   install --group=puppet --owner=puppet -d /var/lib/puppet/jruby-gems
-   mkdir -p /etc/puppet/manifests
 }
 
 # Debian based pre-installation tasks.
@@ -213,27 +213,46 @@ function task_preinst_deb {
         addgroup --quiet --system puppet; \
         usermod -g puppet puppet; \
     fi
-    install --group=puppet --owner=puppet -d /var/lib/puppet/jruby-gems
-    mkdir -p /var/run/puppet
-    mkdir -p /etc/puppet/manifests
 }
 
 # Debian based post-installation tasks.
 function task_postinst_deb {
     task postinst_permissions
+    /opt/puppetlabs/puppet/bin/puppet config set --section master vardir  /opt/puppetlabs/server/data/puppetserver
+    /opt/puppetlabs/puppet/bin/puppet config set --section master logdir  /var/log/puppetlabs/puppetserver
+    /opt/puppetlabs/puppet/bin/puppet config set --section master rundir  /var/run/puppetlabs/puppetserver
+    /opt/puppetlabs/puppet/bin/puppet config set --section master pidfile /var/run/puppetlabs/puppetserver/puppetserver.pid
+    /opt/puppetlabs/puppet/bin/puppet config set --section master codedir /etc/puppetlabs/code
+    usermod --home /opt/puppetlabs/server/data/puppetserver puppet
+    install --directory --owner=puppet --group=puppet --mode=775 /opt/puppetlabs/server/data
+    install --directory /etc/puppetlabs/puppet/ssl
+    chown -R puppet:puppet /etc/puppetlabs/puppet/ssl
+    find /etc/puppetlabs/puppet/ssl -type d -print0 | xargs -0 chmod 770
 }
 
 # RPM based post-installation tasks.
 function task_postinst_redhat {
     : # Null command in case additional_postinst is empty
+    /opt/puppetlabs/puppet/bin/puppet config set --section master vardir  /opt/puppetlabs/server/data/puppetserver
+    /opt/puppetlabs/puppet/bin/puppet config set --section master logdir  /var/log/puppetlabs/puppetserver
+    /opt/puppetlabs/puppet/bin/puppet config set --section master rundir  /var/run/puppetlabs/puppetserver
+    /opt/puppetlabs/puppet/bin/puppet config set --section master pidfile /var/run/puppetlabs/puppetserver/puppetserver.pid
+    /opt/puppetlabs/puppet/bin/puppet config set --section master codedir /etc/puppetlabs/code
+    usermod --home /opt/puppetlabs/server/data/puppetserver puppet
+    install --directory --owner=puppet --group=puppet --mode=775 /opt/puppetlabs/server/data
+    install --directory /etc/puppetlabs/puppet/ssl
+    chown -R puppet:puppet /etc/puppetlabs/puppet/ssl
+    find /etc/puppetlabs/puppet/ssl -type d -print0 | xargs -0 chmod 770
 }
 
 # Global post installation permissions setup. Not to be used by Redhat
 # during package based installation, as this is done by the RPM itself
 # by the %files definitions
 function task_postinst_permissions {
-    chown puppet:puppet /var/log/puppetserver
-    chmod 700 /var/log/puppetserver
+    chown puppet:puppet /var/log/puppetlabs/puppetserver
+    chmod 700 /var/log/puppetlabs/puppetserver
+    chown puppet:puppet $app_data
+    chmod 775 $app_data
 }
 
 # Install logrotate (usually el7, fedora 16 and above)
@@ -252,23 +271,16 @@ function task_logrotate_legacy {
 # Misc functions #
 ##################
 
-# Print output only if EZ_VERBOSE is set
-function debug_echo {
-    if [ -n "${EZ_VERBOSE}" ]; then
-        echo $@
-    fi
-}
-
 # Do basic OS detection using facter.
 function osdetection {
     OSFAMILY=`facter osfamily`
     MAJREV=`facter operatingsystemmajrelease`
 
-    debug_echo "OS Detection results"
-    debug_echo
-    debug_echo "OSFAMILY: ${OSFAMILY}"
-    debug_echo "MAJREV: ${MAJREV}"
-    debug_echo
+    echo "OS Detection results"
+    echo
+    echo "OSFAMILY: ${OSFAMILY}"
+    echo "MAJREV: ${MAJREV}"
+    echo
 }
 
 # Run a task
@@ -277,7 +289,7 @@ function osdetection {
 function task {
     local task=$1
     shift
-    debug_echo "Running task ${task} ..."
+    echo "Running task ${task} ..."
     eval task_$task $@
 }
 
