@@ -5,18 +5,10 @@ require 'net/http'
 require 'base64'
 
 require 'java'
-SyncHttpClient = com.puppetlabs.http.client.Sync
-java_import com.puppetlabs.http.client.SimpleRequestOptions
+java_import com.puppetlabs.http.client.RequestOptions
+java_import com.puppetlabs.http.client.ClientOptions
 java_import com.puppetlabs.http.client.ResponseBodyType
-
-class Puppet::Server::HttpClientError < SocketError
-  attr_reader :cause
-
-  def initialize(message, cause = nil)
-    super(message)
-    @cause = cause
-  end
-end
+SyncHttpClient = com.puppetlabs.http.client.Sync
 
 class Puppet::Server::HttpClient
 
@@ -29,10 +21,7 @@ class Puppet::Server::HttpClient
   # Store a java HashMap of settings related to the http client
   def self.initialize_settings(settings)
     @settings = settings.select { |k,v|
-      ["ssl_protocols",
-       "cipher_suites",
-       "http_connect_timeout_milliseconds",
-       "http_idle_timeout_milliseconds"].include? k
+      ["ssl_protocols", "cipher_suites"].include? k
     }
   end
 
@@ -59,61 +48,45 @@ class Puppet::Server::HttpClient
       end
 
       # http://en.wikipedia.org/wiki/Basic_access_authentication#Client_side
-      encoded = Base64.strict_encode64("#{credentials[:user]}:#{credentials[:password]}")
-      headers["Authorization"] = "Basic #{encoded}"
+      headers["Authorization"] =
+          "Basic #{Base64.strict_encode64 "#{credentials[:user]}:#{credentials[:password]}"}"
     end
 
-    request_options = SimpleRequestOptions.new(build_url(url))
+    # Ensure multiple requests are not made on the same connection
+    headers["Connection"] = "close"
+
+    request_options = RequestOptions.new(build_url(url))
     request_options.set_headers(headers)
     request_options.set_as(ResponseBodyType::TEXT)
     request_options.set_body(body)
-    configure_timeouts(request_options)
-    configure_ssl(request_options)
-    response = client_post(request_options)
+    response = self.class.client_post(request_options)
     ruby_response(response)
   end
 
   def get(url, headers)
-    request_options = SimpleRequestOptions.new(build_url(url))
+    # Ensure multiple requests are not made on the same connection
+    headers["Connection"] = "close"
+
+    request_options = RequestOptions.new(build_url(url))
     request_options.set_headers(headers)
     request_options.set_as(ResponseBodyType::TEXT)
-    configure_timeouts(request_options)
-    configure_ssl(request_options)
-    response = client_get(request_options)
+    response = self.class.client_get(request_options)
     ruby_response(response)
   end
 
+  def self.terminate
+    unless self.client.nil?
+      self.client.close
+    end
+  end
+
   private
 
-  def client_get(request_options)
-    SyncHttpClient.get(request_options)
-  rescue Java::ComPuppetlabsHttpClient::HttpClientException => e
-    raise Puppet::Server::HttpClientError.new(e.message, e)
-  end
-
-  def client_post(request_options)
-    SyncHttpClient.post(request_options)
-  rescue Java::ComPuppetlabsHttpClient::HttpClientException => e
-    raise Puppet::Server::HttpClientError.new(e.message, e)
-  end
-
-  def configure_timeouts(request_options)
-    settings = self.class.settings
-
-    if settings.has_key?("http_connect_timeout_milliseconds")
-      request_options.set_connect_timeout_milliseconds(settings["http_connect_timeout_milliseconds"])
-    end
-
-    if settings.has_key?("http_idle_timeout_milliseconds")
-      request_options.set_socket_timeout_milliseconds(settings["http_idle_timeout_milliseconds"])
-    end
-  end
-
-  def configure_ssl(request_options)
-    return unless @use_ssl
+  def self.configure_ssl(request_options)
     request_options.set_ssl_context(Puppet::Server::Config.ssl_context)
 
-    settings = self.class.settings
+    settings = self.settings
+
     if settings.has_key?("ssl_protocols")
       request_options.set_ssl_protocols(settings["ssl_protocols"])
     end
@@ -141,11 +114,9 @@ class Puppet::Server::HttpClient
     clazz = ruby_response_class(response.status.to_s)
     result = clazz.new(nil, response.status.to_s, nil)
     result.body = response.body
-    # TODO: this is nasty, nasty.  But apparently there is no way to create
+    # This is nasty, nasty.  But apparently there is no way to create
     # an instance of Net::HttpResponse from outside of the library and have
     # the body be readable, unless you do stupid things like this.
-    # We need to figure out how to add some spec tests to make sure that this
-    # fragile thing doesn't break in a future version of jruby. (PE-3356)
     result.instance_variable_set(:@read, true)
 
     response.headers.each do |k,v|
@@ -154,4 +125,22 @@ class Puppet::Server::HttpClient
     result
   end
 
-end
+  def self.create_client
+    client_options = ClientOptions.new
+    self.configure_ssl(client_options)
+    SyncHttpClient.createClient(client_options)
+  end
+
+  def self.client
+    @client ||= create_client
+  end
+
+  def self.client_post(request_options)
+    self.client.post(request_options)
+  end
+
+  def self.client_get(request_options)
+    self.client.get(request_options)
+  end
+
+end
\ No newline at end of file
