@@ -44,7 +44,6 @@ module Puppet::Server::Network::HTTP::Handler
 
   def params(request)
     params = request["params"] || {}
-    params = params_to_ruby(params)
     params = decode_params(params)
     params.merge(client_information(request))
   end
@@ -74,21 +73,4 @@ module Puppet::Server::Network::HTTP::Handler
     result
   end
 
-  def params_to_ruby(params)
-    Hash[params.collect do |key, value|
-      # Values for query string and/or form parameters which are specified
-      # with array-like syntax will be parsed by Ring into a Clojure
-      # PersistentVector, which derives from a Java List.  Need to
-      # translate the Java List into a Ruby Array so that the request
-      # handling logic in Ruby can make use of it.
-
-      # For example, a query string of 'arr=one&arr=two" will be translated
-      # at the Clojure Ring layer into an element with a key of "arr" and
-      # value of '["one", "two"]' as a Clojure PersistentVector.  This
-      # PersistentVector needs to be converted into a Ruby Array before
-      # proceeding with the request processing.
-      [key, value.java_kind_of?(Java::JavaUtil::List) ? value.to_a : value]
-    end]
-  end
-
 end
\ No newline at end of file
