@@ -1,9 +1,7 @@
 (ns puppetlabs.puppetserver.certificate-authority
   (:import [org.apache.commons.io IOUtils]
            [java.util Date]
-           [java.io InputStream ByteArrayOutputStream ByteArrayInputStream File]
-           [java.nio.file Files Paths LinkOption]
-           [java.nio.file.attribute FileAttribute PosixFilePermissions])
+           [java.io InputStream ByteArrayOutputStream ByteArrayInputStream])
   (:require [me.raynes.fs :as fs]
             [schema.core :as schema]
             [clojure.string :as str]
@@ -29,11 +27,10 @@
   {:certdir        schema/Str
    :dns-alt-names  schema/Str
    :hostcert       schema/Str
+   :hostcrl        schema/Str
    :hostprivkey    schema/Str
    :hostpubkey     schema/Str
-   :keylength      schema/Int
    :localcacert    schema/Str
-   :privatekeydir schema/Str
    :requestdir     schema/Str
    :csr-attributes schema/Str})
 
@@ -60,8 +57,6 @@
    :ca-ttl                   schema/Int
    :cert-inventory           schema/Str
    :csrdir                   schema/Str
-   :keylength                schema/Int
-   :manage-internal-file-permissions schema/Bool
    :ruby-load-path           [schema/Str]
    :signeddir                schema/Str
    :serial                   schema/Str})
@@ -125,10 +120,23 @@
 (def puppet-short-names
   "A mapping of Puppet extension short names to their OIDs. These appear in
   csr_attributes.yaml."
-  {:pp_uuid          "1.3.6.1.4.1.34380.1.1.1"
-   :pp_instance_id   "1.3.6.1.4.1.34380.1.1.2"
-   :pp_image_name    "1.3.6.1.4.1.34380.1.1.3"
-   :pp_preshared_key "1.3.6.1.4.1.34380.1.1.4"})
+  {:pp_uuid             "1.3.6.1.4.1.34380.1.1.1"
+   :pp_instance_id      "1.3.6.1.4.1.34380.1.1.2"
+   :pp_image_name       "1.3.6.1.4.1.34380.1.1.3"
+   :pp_preshared_key    "1.3.6.1.4.1.34380.1.1.4"
+   :pp_cost_center      "1.3.6.1.4.1.34380.1.1.5"
+   :pp_product          "1.3.6.1.4.1.34380.1.1.6"
+   :pp_project          "1.3.6.1.4.1.34380.1.1.7"
+   :pp_application      "1.3.6.1.4.1.34380.1.1.8"
+   :pp_service          "1.3.6.1.4.1.34380.1.1.9"
+   :pp_employee         "1.3.6.1.4.1.34380.1.1.10"
+   :pp_created_by       "1.3.6.1.4.1.34380.1.1.11"
+   :pp_environment      "1.3.6.1.4.1.34380.1.1.12"
+   :pp_role             "1.3.6.1.4.1.34380.1.1.13"
+   :pp_software_version "1.3.6.1.4.1.34380.1.1.14"
+   :pp_department       "1.3.6.1.4.1.34380.1.1.15"
+   :pp_cluster          "1.3.6.1.4.1.34380.1.1.16"
+   :pp_provisioner      "1.3.6.1.4.1.34380.1.1.17"})
 
 (def netscape-comment-value
   "Standard value applied to the Netscape Comment extension for certificates"
@@ -136,63 +144,15 @@
 
 (def required-master-files
   "The set of SSL files that are required on the master."
-  #{:hostprivkey :hostcert})
+  [:hostprivkey :hostcert])
 
 (def required-ca-files
   "The set of SSL related files that are required on the CA."
-  #{:cacert :cacrl :cakey :cert-inventory :serial})
+  [:cacert :cacrl :cakey :cert-inventory :serial])
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Internal
 
-(def private-key-perms
-  "Posix permissions for all private keys on disk."
-  "rw-r-----")
-
-(def private-key-dir-perms
-  "Posix permissions for the private key directory on disk."
-  "rwxr-x---")
-
-(def empty-string-array
-  "The API for Paths/get requires a string array which is empty for all of the
-  needs of this namespace. "
-  (into-array String []))
-
-(defn get-path-obj
-  "Create a Path object from the provided path."
-  [path]
-  (Paths/get path empty-string-array))
-
-(schema/defn set-file-perms :- File
-  "Set the provided permissions on the given path. The permissions string is in
-  the form of the standard 9 character posix format, ie \"rwxr-xr-x\"."
-  [path :- schema/Str
-   permissions :- schema/Str]
-  (-> (get-path-obj path)
-      (Files/setPosixFilePermissions
-        (PosixFilePermissions/fromString permissions))
-      (.toFile)))
-
-(schema/defn get-file-perms :- schema/Str
-  "Returns the currently set permissions of the given file path."
-  [path :- schema/Str]
-  (-> (get-path-obj path)
-      (Files/getPosixFilePermissions (into-array LinkOption []))
-      PosixFilePermissions/toString))
-
-(schema/defn create-file-with-perms :- File
-  "Create a new empty file which has the provided posix file permissions. The
-  permissions string is in the form of the standard 9 character posix format. "
-  [path :- schema/Str
-   permissions :- schema/Str]
-  (let [perms-set (PosixFilePermissions/fromString permissions)]
-    (-> (get-path-obj path)
-        (Files/createFile
-         (into-array FileAttribute
-                     [(PosixFilePermissions/asFileAttribute perms-set)]))
-        (Files/setPosixFilePermissions perms-set)
-        (.toFile))))
-
 (schema/defn cert-validity-dates :- {:not-before Date :not-after Date}
   "Calculate the not-before & not-after dates that define a certificate's
    period of validity. The value of `ca-ttl` is expected to be in seconds,
@@ -201,7 +161,7 @@
   [ca-ttl :- schema/Int]
   (let [now        (time/now)
         not-before (time/minus now (time/days 1))
-        not-after  (time/plus now (time/seconds ca-ttl))]
+        not-after  (time/plus now (time/secs ca-ttl))]
     {:not-before (.toDate not-before)
      :not-after  (.toDate not-after)}))
 
@@ -216,8 +176,6 @@
           :autosign
           :ca-name
           :ca-ttl
-          :keylength
-          :manage-internal-file-permissions
           :ruby-load-path))
 
 (schema/defn settings->ssldir-paths
@@ -225,7 +183,7 @@
    paths. These paths are necessary during initialization for determining what
    needs to be created and where."
   [master-settings :- MasterSettings]
-  (dissoc master-settings :dns-alt-names :csr-attributes :keylength))
+  (dissoc master-settings :dns-alt-names :csr-attributes))
 
 (defn path-to-cert
   "Return a path to the `subject`s certificate file under the `signeddir`."
@@ -404,14 +362,15 @@
 (schema/defn generate-ssl-files!
   "Given the CA settings, generate and write to disk all of the necessary
   SSL files for the CA. Any existing files will be replaced."
-  [ca-settings :- CaSettings]
+  [ca-settings :- CaSettings
+   keylength :- schema/Int]
   (log/debug (str "Initializing SSL for the CA; settings:\n"
                   (ks/pprint-to-string ca-settings)))
   (create-parent-directories! (vals (settings->cadir-paths ca-settings)))
   (-> ca-settings :csrdir fs/file ks/mkdirs!)
   (-> ca-settings :signeddir fs/file ks/mkdirs!)
   (initialize-serial-file! (:serial ca-settings))
-  (let [keypair     (utils/generate-key-pair (:keylength ca-settings))
+  (let [keypair     (utils/generate-key-pair keylength)
         public-key  (utils/get-public-key keypair)
         private-key (utils/get-private-key keypair)
         x500-name   (utils/cn (:ca-name ca-settings))
@@ -434,8 +393,7 @@
                                         public-key)]
     (write-cert-to-inventory! cacert (:cert-inventory ca-settings))
     (utils/key->pem! public-key (:capub ca-settings))
-    (utils/key->pem! private-key
-      (create-file-with-perms (:cakey ca-settings) private-key-perms))
+    (utils/key->pem! private-key (:cakey ca-settings))
     (utils/cert->pem! cacert (:cacert ca-settings))
     (utils/crl->pem! cacrl (:cacrl ca-settings))))
 
@@ -582,18 +540,18 @@
    the master. Any existing files will be replaced."
   [settings :- MasterSettings
    certname :- schema/Str
-   ca-settings :- CaSettings]
+   ca-settings :- CaSettings
+   keylength :- schema/Int]
   (log/debug (str "Initializing SSL for the Master; settings:\n"
                   (ks/pprint-to-string settings)))
   (create-parent-directories! (vals (settings->ssldir-paths settings)))
-  (set-file-perms (:privatekeydir settings) private-key-dir-perms)
   (-> settings :certdir fs/file ks/mkdirs!)
   (-> settings :requestdir fs/file ks/mkdirs!)
   (let [ca-cert        (utils/pem->cert (:cacert ca-settings))
         ca-public-key  (.getPublicKey ca-cert)
         ca-private-key (utils/pem->private-key (:cakey ca-settings))
         next-serial    (next-serial-number! (:serial ca-settings))
-        keypair        (utils/generate-key-pair (:keylength settings))
+        keypair        (utils/generate-key-pair keylength)
         public-key     (utils/get-public-key keypair)
         extensions     (create-master-extensions certname
                                                  public-key
@@ -612,8 +570,7 @@
                                                extensions)]
     (write-cert-to-inventory! hostcert (:cert-inventory ca-settings))
     (utils/key->pem! public-key (:hostpubkey settings))
-    (utils/key->pem! private-key
-     (create-file-with-perms (:hostprivkey settings) private-key-perms))
+    (utils/key->pem! private-key (:hostprivkey settings))
     (utils/cert->pem! hostcert (:hostcert settings))
     (utils/cert->pem! hostcert
                       (path-to-cert (:signeddir ca-settings) certname))))
@@ -622,9 +579,12 @@
   "Given configuration settings, certname, and CA settings, ensure all
    necessary SSL files exist on disk by regenerating all of them if any
    are found to be missing."
-  [settings :- MasterSettings
+  ([settings certname ca-settings]
+     (initialize-master-ssl! settings certname ca-settings utils/default-key-length))
+  ([settings :- MasterSettings
     certname :- schema/Str
-    ca-settings :- CaSettings]
+    ca-settings :- CaSettings
+    keylength :- schema/Int]
      (let [required-files (-> (settings->ssldir-paths settings)
                               (select-keys required-master-files)
                               (vals))]
@@ -633,16 +593,16 @@
          (let [{found   true
                 missing false} (group-by fs/exists? required-files)]
            (if (= required-files missing)
-             (generate-master-ssl-files! settings certname ca-settings)
-             (throw (partial-state-error "master" found missing)))))))
+             (generate-master-ssl-files! settings certname ca-settings keylength)
+             (throw (partial-state-error "master" found missing))))))))
 
 (schema/defn ^:always-validate retrieve-ca-cert!
-  "Given configuration settings and CA settings, ensure a local copy of the CA
-  cert is available on disk.  cacert is the base CA cert file to copy from and
-  localcacert is where that the CA cert file should be copied to."
+  "Ensure a local copy of the CA cert is available on disk.  cacert is the base
+  CA cert file to copy from and localcacert is where the CA cert file should be
+  copied to."
   ([cacert :- schema/Str
     localcacert :- schema/Str]
-   (if (and (fs/exists? cacert) (not (fs/exists? localcacert)))
+   (if (fs/exists? cacert)
      (do
        (ks/mkdirs! (fs/parent localcacert))
        (fs/copy cacert localcacert))
@@ -655,6 +615,16 @@
                      cacert
                      ") to copy it from")))))))
 
+(schema/defn ^:always-validate retrieve-ca-crl!
+  "Ensure a local copy of the CA CRL, if one exists, is available on disk.
+  cacrl is the base CRL file to copy from and localcacrl is where the CRL file
+  should be copied to."
+  ([cacrl :- schema/Str
+    localcacrl :- schema/Str]
+    (when (fs/exists? cacrl)
+      (ks/mkdirs! (fs/parent localcacrl))
+      (fs/copy cacrl localcacrl))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Autosign
 
@@ -749,14 +719,14 @@
   "Given the configuration map from the Puppet Server config
    service return a map with of all the CA settings.
    Throws an exception if any required configuration is not found."
-  [{:keys [puppet-server os-settings certificate-authority]}]
+  [{:keys [puppet-server jruby-puppet certificate-authority]}]
   (if-not certificate-authority
     (throw (IllegalStateException.
             (str "Missing required configuration for CA; "
                  "certificate-authority: { certificate-status: { client-whitelist: [...] } } "
                  "not found in puppet-server.conf"))))
   (-> (select-keys puppet-server (keys CaSettings))
-      (assoc :ruby-load-path (:ruby-load-path os-settings))
+      (assoc :ruby-load-path (:ruby-load-path jruby-puppet))
       (assoc :access-control certificate-authority)))
 
 (schema/defn ^:always-validate
@@ -859,9 +829,7 @@
   [subject :- schema/Str
    csr :- CertificateRequest
    csrdir :- schema/Str]
-  (let [csr-path (path-to-cert-request csrdir subject)]
-    (log/debugf "Saving CSR to '%s'" csr-path)
-    (utils/obj->pem! csr csr-path)))
+  (utils/obj->pem! csr (path-to-cert-request csrdir subject)))
 
 (schema/defn validate-duplicate-cert-policy!
   "Throw a slingshot exception if allow-duplicate-certs is false
@@ -946,39 +914,27 @@
       (when-not (fs/exists? path)
         (ks/mkdirs! path)))))
 
-(schema/defn ensure-ca-file-perms!
-  "Ensure that the CA's private key file has the correct permissions set. If it
-  does not, then correct them."
-  [settings :- CaSettings]
-  (let [ca-p-key (:cakey settings)
-        cur-perms (get-file-perms ca-p-key)]
-    (when-not (= private-key-perms cur-perms)
-      (set-file-perms ca-p-key private-key-perms)
-      (log/warnf (str "The private CA key at '%s' was found to have the wrong "
-                      "permissions set as '%s'. This has been corrected to '%s'.")
-                 ca-p-key cur-perms private-key-perms))))
-
 (schema/defn ^:always-validate
   initialize!
-  "Given the CA configuration settings, ensure that all
-   required SSL files exist. If all files exist,
+  "Given the CA configuration settings and an optional keylength,
+   ensure that all required SSL files exist. If all files exist,
    new ones will not be generated. If only some are found
    (but others are missing), an exception is thrown."
-  [settings :- CaSettings]
-    (ensure-directories-exist! settings)
-    (let [required-files (-> (settings->cadir-paths settings)
-                            (select-keys required-ca-files)
-                            (vals))]
-     (if (every? fs/exists? required-files)
-       (do
+  ([settings]
+     (initialize! settings utils/default-key-length))
+  ([settings :- CaSettings
+    keylength :- schema/Int]
+     (ensure-directories-exist! settings)
+     (let [required-files (-> (settings->cadir-paths settings)
+                              (select-keys required-ca-files)
+                              (vals))]
+       (if (every? fs/exists? required-files)
          (log/info "CA already initialized for SSL")
-         (when (:manage-internal-file-permissions settings)
-           (ensure-ca-file-perms! settings)))
-       (let [{found   true
-              missing false} (group-by fs/exists? required-files)]
-         (if (= required-files missing)
-           (generate-ssl-files! settings)
-           (throw (partial-state-error "CA" found missing)))))))
+         (let [{found   true
+                missing false} (group-by fs/exists? required-files)]
+           (if (= required-files missing)
+             (generate-ssl-files! settings keylength)
+             (throw (partial-state-error "CA" found missing))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; certificate_status endpoint
