@@ -1,20 +1,19 @@
 (ns puppetlabs.services.ca.certificate-authority-core
-  (:import [java.io InputStream]
-           (clojure.lang IFn))
+  (:import  [java.io InputStream])
   (:require [puppetlabs.puppetserver.certificate-authority :as ca]
             [puppetlabs.puppetserver.ringutils :as ringutils]
-            [puppetlabs.puppetserver.liberator-utils :as utils]
-            [puppetlabs.comidi :as comidi :refer [GET ANY PUT]]
+            [puppetlabs.puppetserver.liberator-utils :as liberator-utils]
             [slingshot.slingshot :as sling]
             [clojure.tools.logging :as log]
             [clojure.java.io :as io]
             [clojure.string :as string]
             [schema.core :as schema]
             [cheshire.core :as cheshire]
+            [compojure.core :as compojure :refer [GET ANY PUT]]
+            [compojure.route :as route]
             [liberator.core :refer [defresource]]
-            ;[liberator.dev :as liberator-dev]
             [liberator.representation :as representation]
-            [ring.util.request :as request]
+            [liberator.dev :as liberator-dev]
             [ring.util.response :as rr]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -55,7 +54,7 @@
       (rr/content-type "text/plain")))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Web app
+;;; Compojure app
 
 (defn try-to-parse
   [body]
@@ -91,7 +90,7 @@
 
 (defn content-type-valid?
   [context]
-  (let [content-type (request/content-type (:request context))]
+  (let [content-type (get-in context [:request :headers "content-type"])]
     (or
       (nil? content-type)
       (media-types content-type))))
@@ -113,22 +112,6 @@
         (assoc :status 200)
         (representation/ring-response))))
 
-(defn as-plain-text-response
-  "Create a ring response based on the response info in the supplied context
-   and a specific message.  The message is assumed to be plain text and so is
-   marked with a 'text/plain; charset=UTF-8' Content-Type header.  This is
-   needed for cases where liberator would not mark the Content-Type in the
-   response as 'text/plain' on its own, which could otherwise result in the
-   underlying webserver dumbly constructing the Content-Type as
-   ';charset=UTF-8'.  A Content-Type with a charset and no MIME value would be
-   problematic for some clients to interpret."
-  [context message]
-  (-> message
-    (representation/as-response context)
-    (assoc :status (:status context))
-    (assoc-in [:headers "Content-Type"] "text/plain; charset=UTF-8")
-    (representation/ring-response)))
-
 (defresource certificate-status
   [subject settings]
   :allowed-methods [:get :put :delete]
@@ -175,11 +158,9 @@
 
   :handle-conflict
   (fn [context]
-    (as-plain-text-response context (::conflict context)))
+    (::conflict context))
 
-  :handle-exception
-  (fn [context]
-    (as-plain-text-response context (utils/exception-handler context)))
+  :handle-exception liberator-utils/exception-handler
 
   :handle-not-implemented
   (fn [context]
@@ -192,8 +173,10 @@
       ; which makes the most sense in general - see
       ; https://github.com/clojure-liberator/liberator/pull/120
       ; ... but in our case, a 404 definitely makes more sense.
-      (-> (assoc context :status 404)
-        (as-plain-text-response "Invalid certificate subject."))))
+      (-> "Invalid certificate subject."
+          (representation/as-response context)
+          (assoc :status 404)
+          (representation/ring-response))))
 
   :handle-ok
   (fn [context]
@@ -246,9 +229,7 @@
 
   :available-media-types media-types
 
-  :handle-exception
-  (fn [context]
-    (as-plain-text-response context (utils/exception-handler context)))
+  :handle-exception liberator-utils/exception-handler
 
   :handle-ok
   (fn [context]
@@ -256,28 +237,46 @@
       (ca/get-certificate-statuses settings)
       (as-json-or-pson context))))
 
-(schema/defn ^:always-validate web-routes :- comidi/BidiRoute
+(schema/defn v1-routes
   [ca-settings :- ca/CaSettings]
-  (comidi/context ["/" :environment]
-    (ANY ["/certificate_status/" :subject] [subject]
+  (compojure/routes
+    (ANY "/certificate_status/:subject" [subject]
          (certificate-status subject ca-settings))
-    (ANY ["/certificate_statuses/" :ignored-but-required] []
+    (ANY "/certificate_statuses/:ignored-but-required" [do-not-use]
          (certificate-statuses ca-settings))
-    (GET ["/certificate/" :subject] [subject]
+    (GET "/certificate/:subject" [subject]
          (handle-get-certificate subject ca-settings))
-    (comidi/context ["/certificate_request/" :subject]
-      (GET [""] [subject]
-           (handle-get-certificate-request subject ca-settings))
-      (PUT [""] [subject :as {body :body}]
-           (handle-put-certificate-request! subject body ca-settings)))
-    (GET ["/certificate_revocation_list/" :ignored-node-name] []
+    (compojure/context "/certificate_request/:subject" [subject]
+                       (GET "/" []
+                            (handle-get-certificate-request subject ca-settings))
+                       (PUT "/" {body :body}
+                            (handle-put-certificate-request! subject body ca-settings)))
+    (GET "/certificate_revocation_list/:ignored-node-name" []
          (handle-get-certificate-revocation-list ca-settings))))
 
+(defn routes
+  [ca-settings]
+  (compojure/routes
+    (compojure/context "/v1" [] (v1-routes ca-settings))
+    (route/not-found "Not Found")))
+
+(defn wrap-with-puppet-version-header
+  "Function that returns a middleware that adds an
+  X-Puppet-Version header to the response."
+  [handler version]
+  (fn [request]
+    (let [response (handler request)]
+      ; Our compojure app returns nil responses sometimes.
+      ; In that case, don't add the header.
+      (when response
+        (rr/header response "X-Puppet-Version" version)))))
+
 (schema/defn ^:always-validate
-  wrap-middleware :- IFn
-  [handler :- IFn
+  build-ring-handler
+  [ca-settings :- ca/CaSettings
    puppet-version :- schema/Str]
-  (-> handler
-    ;(liberator-dev/wrap-trace :header)           ; very useful for debugging!
-    (ringutils/wrap-with-puppet-version-header puppet-version)
-    (ringutils/wrap-response-logging)))
+  (-> (routes ca-settings)
+      ;(liberator-dev/wrap-trace :header)           ; very useful for debugging!
+      (ringutils/wrap-exception-handling)
+      (wrap-with-puppet-version-header puppet-version)
+      (ringutils/wrap-response-logging)))
