@@ -4,7 +4,8 @@
             [puppetlabs.puppetserver.certificate-authority :as ca]
             [puppetlabs.services.ca.certificate-authority-core :as core]
             [puppetlabs.services.protocols.ca :refer [CaService]]
-            [puppetlabs.comidi :as comidi]))
+            [compojure.core :as compojure]
+            [me.raynes.fs :as fs]))
 
 (tk/defservice certificate-authority-service
   CaService
@@ -19,10 +20,7 @@
      (log/info "CA Service adding a ring handler")
      (add-ring-handler
        this
-       (-> (core/web-routes settings)
-           (#(comidi/context path %))
-           comidi/routes->handler
-           (core/wrap-middleware puppet-version))))
+      (compojure/context path [] (core/build-ring-handler settings puppet-version))))
    context)
 
   (initialize-master-ssl!
@@ -33,4 +31,9 @@
   (retrieve-ca-cert!
     [this localcacert]
     (ca/retrieve-ca-cert! (get-in-config [:puppet-server :cacert])
-                          localcacert)))
+                          localcacert))
+
+  (retrieve-ca-crl!
+    [this localcacrl]
+    (ca/retrieve-ca-crl! (get-in-config [:puppet-server :cacrl])
+                         localcacrl)))
