@@ -3,4 +3,5 @@
 (defprotocol CaService
   "Describes the functionality of the CA service."
   (initialize-master-ssl! [this master-settings certname])
-  (retrieve-ca-cert! [this master-settings]))
+  (retrieve-ca-cert! [this localcacert])
+  (retrieve-ca-crl! [this localcacrl]))
